@@ -1,28 +1,28 @@
 #' Upload a file to RPubs
 #'
 #' This function publishes a file to rpubs.com. If the upload succeeds a
-#' list that includes an \code{id} and \code{continueUrl} is returned. A browser
-#' should be opened to the \code{continueUrl} to complete publishing of the
+#' list that includes an `id` and `continueUrl` is returned. A browser
+#' should be opened to the `continueUrl` to complete publishing of the
 #' document. If an error occurs then a diagnostic message is returned in the
-#' \code{error} element of the list.
+#' `error` element of the list.
 #'
 #' @param title The title of the document.
 #' @param contentFile The path to the content file to upload.
 #' @param originalDoc The document that was rendered to produce the
-#'   \code{contentFile}. May be \code{NULL} if the document is not known.
+#'   `contentFile`. May be `NULL` if the document is not known.
 #' @param id If this upload is an update of an existing document then the id
 #'   parameter should specify the document id to update. Note that the id is
 #'   provided as an element of the list returned by successful calls to
-#'   \code{rpubsUpload}.
+#'   `rpubsUpload`.
 #' @param properties A named list containing additional document properties
 #'   (RPubs doesn't currently expect any additional properties, this parameter
 #'   is reserved for future use).
 #'
 #' @return A named list. If the upload was successful then the list contains a
-#'   \code{id} element that can be used to subsequently update the document as
-#'   well as a \code{continueUrl} element that provides a URL that a browser
+#'   `id` element that can be used to subsequently update the document as
+#'   well as a `continueUrl` element that provides a URL that a browser
 #'   should be opened to in order to complete publishing of the document. If the
-#'   upload fails then the list contains an \code{error} element which contains
+#'   upload fails then the list contains an `error` element which contains
 #'   an explanation of the error that occurred.
 #'
 #' @examples
@@ -71,7 +71,7 @@ rpubsUpload <- function(title,
 
     # build package.json
     properties$title = title
-    packageJson <- RJSONIO::toJSON(properties)
+    packageJson <- toJSON(properties)
 
     # create a tempdir to build the package in and copy the files to it
     fileSep <- .Platform$file.sep
@@ -138,7 +138,7 @@ rpubsUpload <- function(title,
 
   # return either id & continueUrl or error
   if (succeeded) {
-    parsedContent <- RJSONIO::fromJSON(content)
+    parsedContent <- jsonlite::fromJSON(content)
     id <- ifelse(isUpdate, id, result$location)
     url <- as.character(parsedContent["continueUrl"])
 
